from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector


class IdentityManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.identity_conn: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", service="identity"
        )

    def get_workspace(self, workspace_id):
        return self.identity_conn.dispatch(
            "Workspace.get",
            {"workspace_id": workspace_id},
        )

    def get_trusted_account(self, trusted_account_id):
        return self.identity_conn.dispatch(
            "TrustedAccount.get",
            {"trusted_account_id": trusted_account_id},
        )

    def list_trusted_accounts(self, query):
        return self.identity_conn.dispatch("TrustedAccount.list", {"query": query})

    def get_service_account(self, service_account_id):
        return self.identity_conn.dispatch(
            "ServiceAccount.get",
            {"service_account_id": service_account_id},
        )

    def list_service_accounts(self, query):
        return self.identity_conn.dispatch("ServiceAccount.list", {"query": query})

    def get_project(self, project_id):
        return self.identity_conn.dispatch("Project.get", {"project_id": project_id})

    def list_projects(self, query):
        return self.identity_conn.dispatch("Project.list", {"query": query})



import logging
import json
import consul

from spaceone.core.error import *
from spaceone.core.connector import BaseConnector

__all__ = ['ConsulConnector']
_LOGGER = logging.getLogger(__name__)


class ConsulConnector(BaseConnector):
    """ Consul Backend
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.config = self._validate_config(self.config)

        # No configuration
        if self.config == {}:
            raise ERROR_CONNECTOR_CONFIGURATION(backend='ConsulConnector')

        # Create client
        self.client = consul.Consul(**self.config)

    def _validate_config(self, config):
        """
        Parameter for Consul
        - host, port=8500, token=None, scheme=http, consistency=default, dc=None, verify=True, cert=None
        """
        options = ['host', 'port', 'token', 'scheme', 'consistency', 'dc', 'verify', 'cert']
        result = {}
        for item in options:
            value = config.get(item, None)
            if value:
                result[item] = value
        return result

    @staticmethod
    def _response(response):
        # TODO: error check
        if isinstance(response, dict):
            return True

        return False

    @staticmethod
    def _response_value(response):
        index, data = response
        # TODO: error check
        if 'Value' not in data:
            return False
        value = data['Value'].decode('ascii')
        secret = json.loads(value)
        print("##################")
        print(secret)
        print("-----------------")
        name = secret.get("Name", None)
        secret_string = secret.get("SecretString", None)
        if name == None or secret_string == None:
            _LOGGER.error(f'[_response_value] {secret}')
        return json.loads( secret_string)

    def create_secret(self, secret_id, data):
        secret_params = {
            'Name': secret_id,
            'SecretString': json.dumps(data),
        }
        return self._response(self.client.kv.put(secret_id, json.dumps(secret_params)))

    def delete_secret(self, secret_id):
        response = self.client.kv.delete(secret_id)
        return self._response(response)

    def update_secret(self, secret_id, data):
        self.delete_secret(secret_id)
        response = self.create_secret(secret_id, data)

        return self._response(response)

    def get_secret(self, secret_id):
        return self._response_value(self.client.kv.get(secret_id))



import logging

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.secret.error.custom import *
from spaceone.secret.manager.identity_manager import IdentityManager
from spaceone.secret.manager.secret_manager import SecretManager
from spaceone.secret.model.secret_model import Secret
from spaceone.secret.manager.trusted_secret_manager import TrustedSecretManager
from spaceone.secret.manager.secret_connector_manager import SecretConnectorManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class SecretService(BaseService):
    resource = "Secret"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.secret_mgr: SecretManager = self.locator.get_manager("SecretManager")
        self.identity_mgr: IdentityManager = self.locator.get_manager("IdentityManager")

    @transaction(
        permission="secret:Secret.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["name", "data", "resource_group", "domain_id"])
    def create(self, params):
        """Create secret

        Args:
            params (dict): {
                'name': 'str',                  # required
                'data': 'dict',                 # required
                'schema_id': 'str',
                'tags': 'dict',
                'encrypted': 'bool',
                'encrypt_options': 'dict',
                'trusted_secret_id': 'str',
                'service_account_id': 'str',
                'resource_group': 'str',        # required
                'project_id': 'str',
                'workspace_id': 'str',          # inherited from auth
                'domain_id': 'str'              # inherited from auth (required)
            }

        Returns:
            secret_vo
        """

        resource_group = params["resource_group"]
        domain_id = params["domain_id"]
        workspace_id = params.get("workspace_id")

        # Check permission by resource group
        if resource_group == "PROJECT":
            if "service_account_id" in params:
                service_account_info = self.identity_mgr.get_service_account(
                    params["service_account_id"]
                )

                params["provider"] = service_account_info["provider"]
                params["project_id"] = service_account_info["project_id"]
                params["workspace_id"] = service_account_info["workspace_id"]
            else:
                project_info = self.identity_mgr.get_project(params["project_id"])
                params["workspace_id"] = project_info["workspace_id"]
        elif resource_group == "WORKSPACE":
            self.identity_mgr.get_workspace(params["workspace_id"])
            params["project_id"] = "*"
        else:
            params["workspace_id"] = "*"
            params["project_id"] = "*"

        if "trusted_secret_id" in params:
            if workspace_id:
                workspace_id = [workspace_id, "*"]

            trusted_secret_mgr = self.locator.get_manager("TrustedSecretManager")
            trusted_secret_mgr.get_trusted_secret(
                params["trusted_secret_id"], domain_id, workspace_id
            )

        secret_vo = self.secret_mgr.create_secret(params)

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager(
            "SecretConnectorManager"
        )
        secret_conn_mgr.create_secret(secret_vo.secret_id, params["data"])

        return secret_vo

    @transaction(
        permission="secret:Secret.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["secret_id", "domain_id"])
    def update(self, params):
        """Update secret

        Args:
            params (dict): {
                'secret_id': 'str',         # required
                'name': 'str' ,
                'tags': 'dict',
                'project_id': 'str',
                'workspace_id': 'str',      # inherited from auth
                'domain_id': 'str'          # inherited from auth (required)
                'user_projects': 'list',    # inherited from auth
            }

        Returns:
            secret_vo
        """

        domain_id = params["domain_id"]
        secret_id = params["secret_id"]
        workspace_id = params.get("workspace_id")
        user_projects = params.get("user_projects")

        secret_vo: Secret = self.secret_mgr.get_secret(
            secret_id, domain_id, workspace_id, user_projects
        )

        if secret_vo.resource_group == "PROJECT":
            if project_id := params.get("project_id"):
                self.identity_mgr.get_project(project_id)
            else:
                raise ERROR_PERMISSION_DENIED()

        secret_vo = self.secret_mgr.update_secret_by_vo(params, secret_vo)

        return secret_vo

    @transaction(
        permission="secret:Secret.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["secret_id", "domain_id"])
    def delete(self, params):
        """Delete secret

        Args:
            params (dict): {
                'secret_id': 'str',         # required
                'workspace_id': 'str',      # inherited from auth
                'domain_id': 'str'.         # inherited from auth (required)
                'user_projects': 'list',    # inherited from auth
            }

        Returns:
            None
        """

        secret_id = params["secret_id"]
        domain_id = params["domain_id"]
        workspace_id = params.get("workspace_id")
        user_projects = params.get("user_projects")

        secret_vo = self.secret_mgr.get_secret(
            secret_id, domain_id, workspace_id, user_projects
        )

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager(
            "SecretConnectorManager"
        )
        secret_conn_mgr.delete_secret(secret_id)

        self.secret_mgr.delete_secret_by_vo(secret_vo)

    @transaction(
        permission="secret:Secret.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["secret_id", "data", "domain_id"])
    def update_data(self, params):
        """Update secret data through backend Secret service

        Args:
            params (dict): {
                'secret_id': 'str',             # required
                'schema_id': 'str',
                'data': 'dict',                 # required
                'encrypted': 'bool',
                'encrypt_options': 'dict',
                'workspace_id': 'str',          # inherited from auth
                'domain_id': 'str',             # inherited from auth (required)
                'user_projects': 'list',        # inherited from auth
            }

        Returns:
            secret_data (dict)
        """
        domain_id = params["domain_id"]
        secret_id = params["secret_id"]
        workspace_id = params.get("workspace_id")
        user_projects = params.get("user_projects")
        data = params["data"]

        secret_vo = self.secret_mgr.get_secret(
            secret_id, domain_id, workspace_id, user_projects
        )
        self.secret_mgr.update_secret_by_vo(params, secret_vo)

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager(
            "SecretConnectorManager"
        )
        secret_conn_mgr.update_secret(secret_id, data)

    @transaction(exclude=["authentication", "authorization", "mutation"])
    @check_required(["secret_id", "domain_id"])
    def get_data(self, params):
        """Get secret data through backend Secret service

        Args:
            params (dict): {
                'secret_id': 'str',         # required
                'workspace_id': 'str',      # inherited from auth
                'domain_id': 'str',         # inherited from auth (required)
                'user_projects': 'list',    # inherited from auth
            }

        Returns:
            secret_data (dict)
        """

        secret_id = params["secret_id"]
        domain_id = params["domain_id"]
        workspace_id = params.get("workspace_id")
        user_projects = params.get("user_projects")

        secret_vo: Secret = self.secret_mgr.get_secret(
            secret_id, domain_id, workspace_id, user_projects
        )
        secret_data = self._get_secret_data(secret_id)
        encrypt_options = secret_vo.encrypt_options

        if secret_vo.trusted_secret_id:
            trusted_secret_mgr: TrustedSecretManager = self.locator.get_manager(
                "TrustedSecretManager"
            )
            trusted_secret_vo = trusted_secret_mgr.get_trusted_secret(
                trusted_secret_id=secret_vo.trusted_secret_id, domain_id=domain_id
            )

            self._check_validation_trusted_secret(secret_vo, trusted_secret_vo)

            trusted_secret_data = self._get_secret_data(
                trusted_secret_vo.trusted_secret_id
            )
            trusted_secret_encrypt_options = trusted_secret_vo.encrypt_options

            if secret_vo.encrypted and trusted_secret_vo.encrypted:
                secret_data["trusted_encrypted_data"] = trusted_secret_data[
                    "encrypted_data"
                ]

                encrypt_options.update(
                    {
                        "trusted_encrypted_data_key": trusted_secret_encrypt_options.get(
                            "encrypted_data_key"
                        )
                    }
                )
            elif secret_vo.encrypted is False and trusted_secret_vo.encrypted is False:
                # Merge secret data & trusted secret data
                trusted_secret_data.update(secret_data)
                secret_data = trusted_secret_data

        return {
            "encrypted": secret_vo.encrypted,
            "encrypt_options": encrypt_options,
            "data": secret_data,
        }

    @transaction(
        permission="secret:Secret.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @check_required(["secret_id", "domain_id"])
    def get(self, params):
        """Get secret

        Args:
            params (dict): {
                'secret_id': 'str',         # required
                'workspace_id': 'str',      # inherited from auth
                'domain_id': 'str',         # inherited from auth (required)
                'user_projects': 'list',    # inherited from auth
            }

        Returns:
            secret_vo
        """

        return self.secret_mgr.get_secret(
            params["secret_id"],
            params["domain_id"],
            params.get("workspace_id"),
            params.get("user_projects"),
        )

    @transaction(
        permission="secret:Secret.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @check_required(["domain_id"])
    @append_query_filter(
        [
            "secret_id",
            "name",
            "schema_id",
            "provider",
            "trusted_secret_id",
            "service_account_id",
            "project_id",
            "workspace_id",
            "domain_id",
            "user_projects",
        ]
    )
    @append_keyword_filter(["secret_id", "name", "schema_id", "provider"])
    def list(self, params):
        """List secrets

        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'secret_id': 'str',
                'name': 'str',
                'schema_id': 'str',
                'provider': 'str',
                'trusted_secret_id': 'str',
                'service_account_id': 'str',
                'project_id': 'str',
                'workspace_id': 'str',          # inherited from auth
                'domain_id': 'str',             # inherited from auth (required)
                'user_projects': 'list',        # inherited from auth
            }

        Returns:
            results (list)
            total_count (int)
        """

        query = params.get("query", {})
        secret_vos, total_count = self.secret_mgr.list_secrets(query)

        return secret_vos, total_count

    @transaction(
        permission="secret:Secret.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @check_required(["query", "domain_id"])
    @append_query_filter(["user_projects", "workspace_id", "domain_id"])
    @append_keyword_filter(["secret_id", "name", "schema_id", "provider"])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)', # required
                'workspace_id': 'str',      # inherited from auth
                'domain_id': 'str',         # inherited from auth (required)
                'user_projects': 'list',    # inherited from auth
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get("query", {})
        return self.secret_mgr.stat_secrets(query)

    def _get_secret_data(self, secret_id):
        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager(
            "SecretConnectorManager"
        )
        return secret_conn_mgr.get_secret(secret_id)

    @staticmethod
    def _check_validation_trusted_secret(secret_vo, trusted_secret_vo):
        if secret_vo.encrypted != trusted_secret_vo.encrypted:
            raise ERROR_DIFF_SECRET_AND_TRUSTED_SECRET_ENCRYPTED()

        if secret_vo.encrypted and trusted_secret_vo.encrypted:
            secret_encrypt_options = secret_vo.encrypt_options
            trusted_secret_encrypt_options = trusted_secret_vo.encrypt_options

            secret_encrypt_algorithm = secret_encrypt_options.get("encrypt_algorithm")
            trusted_secret_encrypt_algorithm = trusted_secret_encrypt_options.get(
                "encrypt_algorithm"
            )

            if (
                secret_encrypt_algorithm
                and trusted_secret_encrypt_algorithm
                and secret_encrypt_algorithm == trusted_secret_encrypt_algorithm
            ):
                return True
            else:
                raise ERROR_DIFF_SECRET_AND_TRUSTED_SECRET_ENCRYPTED()

        return True



import logging

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.secret.error.custom import *
from spaceone.secret.manager.identity_manager import IdentityManager
from spaceone.secret.manager.secret_manager import SecretManager
from spaceone.secret.manager.trusted_secret_manager import TrustedSecretManager
from spaceone.secret.manager.secret_connector_manager import SecretConnectorManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class TrustedSecretService(BaseService):
    resource = "TrustedSecret"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.trusted_secret_mgr: TrustedSecretManager = self.locator.get_manager(
            "TrustedSecretManager"
        )
        self.identity_mgr: IdentityManager = self.locator.get_manager("IdentityManager")

    @transaction(
        permission="secret:TrustedSecret.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @check_required(["name", "data", "resource_group", "domain_id"])
    def create(self, params):
        """Create trusted secret

        Args:
            params (dict): {
                'name': 'str',                  # required
                'data': 'dict',                 # required
                'schema_id': 'str',
                'tags': 'dict',
                'encrypted': 'bool',
                'encrypt_options': 'dict',
                'trusted_account_id': 'str',
                'resource_group': 'str',        # required
                'workspace_id': 'str',          # injected from auth
                'domain_id': 'str'              # injected from auth (required)
            }

        Returns:
            trusted_secret_vo
        """

        # Check permission by resource group
        if params["resource_group"] == "WORKSPACE":
            self.identity_mgr.get_workspace(params["workspace_id"])
        else:
            params["workspace_id"] = "*"

        if "trusted_account_id" in params:
            trusted_account_info = self.identity_mgr.get_trusted_account(
                params["trusted_account_id"]
            )
            params["provider"] = trusted_account_info.get("provider")

        trusted_secret_vo = self.trusted_secret_mgr.create_trusted_secret(params)

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager(
            "SecretConnectorManager"
        )

        secret_conn_mgr.create_secret(
            trusted_secret_vo.trusted_secret_id, params["data"]
        )

        return trusted_secret_vo

    @transaction(
        permission="secret:TrustedSecret.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @check_required(["trusted_secret_id", "domain_id"])
    def update(self, params):
        """Update trusted secret

        Args:
            params (dict): {
                'trusted_secret_id': 'str',         # required
                'name': 'str' ,
                'tags': 'dict',
                'workspace_id': 'str',              # injected from auth
                'domain_id': 'str'                  # injected from auth (required)
            }

        Returns:
            trusted_secret_vo
        """

        trusted_secret_id = params["trusted_secret_id"]
        domain_id = params["domain_id"]
        workspace_id = params.get("workspace_id")

        trusted_secret_vo = self.trusted_secret_mgr.get_trusted_secret(
            trusted_secret_id, domain_id, workspace_id
        )
        update_trusted_secret_vo = self.trusted_secret_mgr.update_trusted_secret_by_vo(
            params, trusted_secret_vo
        )

        return update_trusted_secret_vo

    @transaction(
        permission="secret:TrustedSecret.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @check_required(["trusted_secret_id", "domain_id"])
    def delete(self, params):
        """Delete trusted secret

        Args:
            params (dict): {
                'trusted_secret_id': 'str',     # required
                'domain_id': 'str'              # injected from auth (required)
            }

        Returns:
            None
        """

        domain_id = params["domain_id"]
        trusted_secret_id = params["trusted_secret_id"]
        workspace_id = params.get("workspace_id")

        trusted_secret_vo = self.trusted_secret_mgr.get_trusted_secret(
            trusted_secret_id, domain_id, workspace_id
        )

        self._check_related_secret(trusted_secret_id, domain_id)

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager(
            "SecretConnectorManager"
        )
        secret_conn_mgr.delete_secret(trusted_secret_id)

        self.trusted_secret_mgr.delete_trusted_secret_by_vo(trusted_secret_vo)

    @transaction(
        permission="secret:TrustedSecret.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @check_required(["trusted_secret_id", "data", "domain_id"])
    def update_data(self, params):
        """Update trusted secret data through backend Secret service

        Args:
            params (dict): {
                'trusted_secret_id': 'str',     # required
                'schema_id': 'str',
                'data': 'dict',                 # required
                'encrypted': 'bool',
                'encrypt_options': 'dict',
                'workspace_id': 'str',          # injected from auth
                'domain_id': 'str'              # injected from auth (required)
            }

        Returns:
            None
        """

        domain_id = params["domain_id"]
        trusted_secret_id = params["trusted_secret_id"]
        workspace_id = params.get("workspace_id")
        data = params["data"]

        trusted_secret_vo = self.trusted_secret_mgr.get_trusted_secret(
            trusted_secret_id, domain_id, workspace_id
        )
        self.trusted_secret_mgr.update_trusted_secret_by_vo(params, trusted_secret_vo)

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager(
            "SecretConnectorManager"
        )
        secret_conn_mgr.update_secret(trusted_secret_id, data)

    @transaction(
        permission="secret:TrustedSecret.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @check_required(["trusted_secret_id", "domain_id"])
    def get(self, params):
        """Get trusted secret

        Args:
            params (dict): {
                'trusted_secret_id': 'str',         # required
                'workspace_id': 'list',             # injected from auth
                'domain_id': 'str',                 # injected from auth (required)
            }

        Returns:
            trusted_secret_vo
        """

        return self.trusted_secret_mgr.get_trusted_secret(
            params["trusted_secret_id"], params["domain_id"], params.get("workspace_id")
        )

    @transaction(
        permission="secret:TrustedSecret.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @check_required(["domain_id"])
    @append_query_filter(
        [
            "trusted_secret_id",
            "name",
            "schema_id",
            "provider",
            "trusted_account_id",
            "workspace_id",
            "domain_id",
        ]
    )
    @append_keyword_filter(["trusted_secret_id", "name", "schema_id", "provider"])
    def list(self, params):
        """List trusted secrets

        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'trusted_secret_id': 'str',
                'name': 'str',
                'schema_id': 'str',
                'provider': 'str',
                'trusted_account_id': 'str',
                'workspace_id': 'list',             # injected from auth
                'domain_id': 'str',                 # injected from auth (required)
            }

        Returns:
            results (list)
            total_count (int)
        """

        query = params.get("query", {})
        trusted_secret_vos, total_count = self.trusted_secret_mgr.list_trusted_secrets(
            query
        )

        return trusted_secret_vos, total_count

    @transaction(
        permission="secret:TrustedSecret.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @check_required(["query", "domain_id"])
    @append_query_filter(["workspace_id", "domain_id"])
    @append_keyword_filter(["trusted_secret_id", "name", "schema_id", "provider"])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',     # required
                'workspace_id': 'list',         # injected from auth
                'domain_id': 'str',             # injected from auth (required)
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get("query", {})
        return self.trusted_secret_mgr.stat_trusted_secrets(query)

    def _check_related_secret(self, trusted_secret_id, domain_id):
        secret_mgr: SecretManager = self.locator.get_manager("SecretManager")
        secret_vos = secret_mgr.filter_secrets(
            trusted_secret_id=trusted_secret_id, domain_id=domain_id
        )
        if secret_vos.count() > 0:
            raise ERROR_EXIST_RELATED_SECRET(secret_id=secret_vos[0].secret_id)



import logging
import json
import hvac

from spaceone.core.error import *
from spaceone.core.connector import BaseConnector

__all__ = ['VaultConnector']
_LOGGER = logging.getLogger(__name__)


class VaultConnector(BaseConnector):
    """ Vault backend is for develop use
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        vault_url = self.config.get('url')
        token = self.config.get('token')
        if vault_url and token:
            self.client = hvac.Client(url=vault_url)
            self.client.token = token
        else:
            raise ERROR_CONNECTOR_CONFIGURATION(backend='VaultConnector')

    @staticmethod
    def _response(response):
        # TODO: error check
        if isinstance(response, dict):
            return True

        return False

    @staticmethod
    def _response_value(response):
        # TODO: error check
        if 'data' not in response:
            return False
        data = response['data']
        if 'data' not in data:
            return False
        data = data['data']
        data_string = data.get('SecretString', False)
        if data_string:
            return json.loads(data_string)

        return data_string

    def create_secret(self, secret_id, data):
        secret_params = {
            'Name': secret_id,
            'SecretString': json.dumps(data),
        }

        return self._response(self.client.secrets.kv.v2.create_or_update_secret(path=secret_id, secret=secret_params))

    def delete_secret(self, secret_id):
        response = self.client.secrets.kv.delete_metadata_and_all_versions(secret_id)
        return self._response(response)

    def update_secret(self, secret_id, data):
        self.delete_secret(secret_id)
        response = self.create_secret(secret_id, data)

        return self._response(response)

    def get_secret(self, secret_id):
        return self._response_value(self.client.secrets.kv.read_secret_version(path=secret_id))



import logging
import json
import boto3

from spaceone.core.error import *
from spaceone.core.connector import BaseConnector

__all__ = ['AWSSecretManagerConnector']
_LOGGER = logging.getLogger(__name__)


class AWSSecretManagerConnector(BaseConnector):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        aws_access_key_id = self.config.get('aws_access_key_id')
        aws_secret_access_key = self.config.get('aws_secret_access_key')
        region_name = self.config.get('region_name')

        if region_name is None:
            raise ERROR_CONNECTOR_CONFIGURATION(backend='AWSSecretManagerConnector')

        if aws_access_key_id and aws_secret_access_key:
            self.client = boto3.client('secretsmanager', aws_access_key_id=aws_access_key_id,
                                       aws_secret_access_key=aws_secret_access_key, region_name=region_name)
        else:
            self.client = boto3.client('secretsmanager', region_name=region_name)

    @staticmethod
    def _convert_tags(tags):
        return list(map(lambda k: {'Key': k, 'Value': tags[k]}, tags))

    @staticmethod
    def _response(response):
        if 'ResponseMetadata' in response and 'HTTPStatusCode' in response['ResponseMetadata'] and \
                response['ResponseMetadata']['HTTPStatusCode'] == 200:
            return True

        return False

    @staticmethod
    def _response_value(response):
        data_string = response.get('SecretString', False)

        if data_string:
            return json.loads(data_string)

        return data_string

    def create_secret(self, secret_id, data):
        response = self.client.create_secret(
            Name=secret_id,
            SecretString=json.dumps(data)
        )

        return self._response(response)

    def delete_secret(self, secret_id):
        response = self.client.delete_secret(
            SecretId=secret_id,
            ForceDeleteWithoutRecovery=True
        )

        return self._response(response)

    def update_secret(self, secret_id, data):
        response = self.client.put_secret_value(
            SecretId=secret_id,
            SecretString=json.dumps(data)
        )

        return self._response(response)

    def get_secret(self, secret_id):
        return self._response_value(self.client.get_secret_value(SecretId=secret_id))



import logging
import functools
from spaceone.api.secret.v1 import trusted_secret_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.secret.model.trusted_secret_model import TrustedSecret

__all__ = ["TrustedSecretInfo", "TrustedSecretsInfo"]
_LOGGER = logging.getLogger(__name__)


def TrustedSecretInfo(trusted_secret_vo: TrustedSecret, minimal=False):
    info = {
        "trusted_secret_id": trusted_secret_vo.trusted_secret_id,
        "name": trusted_secret_vo.name,
        "schema_id": trusted_secret_vo.schema_id,
        "provider": trusted_secret_vo.provider,
    }

    if minimal is False:
        info.update(
            {
                "tags": change_struct_type(trusted_secret_vo.tags),
                "trusted_account_id": trusted_secret_vo.trusted_account_id,
                "resource_group": trusted_secret_vo.resource_group,
                "workspace_id": trusted_secret_vo.workspace_id,
                "domain_id": trusted_secret_vo.domain_id,
                "created_at": utils.datetime_to_iso8601(trusted_secret_vo.created_at),
            }
        )

    return trusted_secret_pb2.TrustedSecretInfo(**info)


def TrustedSecretsInfo(trusted_secret_vos, total_count, **kwargs):
    results = list(
        map(functools.partial(TrustedSecretInfo, **kwargs), trusted_secret_vos)
    )
    return trusted_secret_pb2.TrustedSecretsInfo(
        results=results, total_count=total_count
    )



from spaceone.secret.manager.secret_manager import SecretManager
from spaceone.secret.manager.trusted_secret_manager import TrustedSecretManager
from spaceone.secret.manager.secret_connector_manager import SecretConnectorManager
from spaceone.secret.manager.identity_manager import IdentityManager



import logging
import functools
from spaceone.api.secret.v1 import secret_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.secret.model.secret_model import Secret

__all__ = ["SecretInfo", "SecretsInfo", "SecretDataInfo"]
_LOGGER = logging.getLogger(__name__)


def SecretDataInfo(secret_data):
    info = {
        "encrypted": secret_data.get("encrypted", False),
        "encrypt_options": change_struct_type(secret_data.get("encrypt_options", {})),
        "data": change_struct_type(secret_data["data"]),
    }

    return secret_pb2.SecretDataInfo(**info)


def SecretInfo(secret_vo: Secret, minimal=False):
    info = {
        "secret_id": secret_vo.secret_id,
        "name": secret_vo.name,
        "schema_id": secret_vo.schema_id,
        "provider": secret_vo.provider,
    }

    if minimal is False:
        info.update(
            {
                "tags": change_struct_type(secret_vo.tags),
                "trusted_secret_id": secret_vo.trusted_secret_id,
                "service_account_id": secret_vo.service_account_id,
                "resource_group": secret_vo.resource_group,
                "project_id": secret_vo.project_id,
                "workspace_id": secret_vo.workspace_id,
                "domain_id": secret_vo.domain_id,
                "created_at": utils.datetime_to_iso8601(secret_vo.created_at),
            }
        )

    return secret_pb2.SecretInfo(**info)


def SecretsInfo(secret_vos, total_count, **kwargs):
    results = list(map(functools.partial(SecretInfo, **kwargs), secret_vos))

    return secret_pb2.SecretsInfo(results=results, total_count=total_count)



from spaceone.core.pygrpc.server import GRPCServer
from spaceone.secret.interface.grpc.secret import Secret
from spaceone.secret.interface.grpc.trusted_secret import TrustedSecret

_all_ = ["app"]

app = GRPCServer()
app.add_service(Secret)
app.add_service(TrustedSecret)
